from imgenx.predictor.base.base_text_to_video import BaseTextToVideo


class AliyunTextToVideo(BaseTextToVideo):

    def __init__(self, model: str, api_key: str):
        self.model = model
        self.api_key = api_key

    def text_to_video(self, prompt: str, resolution: str, ratio: str, duration: int) -> str:
        '''此工具没有实现，确保不要调用此工具'''
        return '此工具没有实现'



import base64
from pathlib import Path
from dashscope import MultiModalConversation

from imgenx.predictor.base.base_image_to_image import BaseImageToImage


class AliyunImageToImage(BaseImageToImage):

    def __init__(self, model: str, api_key: str):
        self.model = model
        self.api_key = api_key

    def image_to_image(self, prompt: str, images: list, size: str):
        '''根据提示词和图片生成图片

        Args:
            prompt (str): 图片描述提示词
            images (List[str]): 输入图片url列表或文件路径列表
            size (str): 图片分辨率
        
        Returns:
            图片url列表
        '''
        messages = [{'role': 'user', 'content': []}]

        for i in images:
            if not str(i).startswith('http'):
                i = self._image_to_base64(i)
            messages[-1]['content'].append({'image': i})

        messages[-1]['content'].append({'text': prompt})

        response = MultiModalConversation.call(
            model=self.model,
            api_key=self.api_key,
            result_format='message',
            stream=False,
            watermark=False,
            negative_prompt="",
            messages=messages
        )

        result = []
        for image_url in response.output.choices[0].message.content:
            result.append(image_url)

        return result

    def _image_to_base64(self, image_path: str) -> str:
        image_path = Path(image_path)

        with open(image_path, 'rb') as image_file:
            base64_image = base64.b64encode(image_file.read()).decode('utf-8')
            base64_image = f'data:image/{image_path.suffix.strip(".")};base64,{base64_image}'

        return base64_image


import base64
from pathlib import Path
import dashscope

from imgenx.predictor.base.base_image_inspector import BaseImageInspector


class AliyunImageInspector(BaseImageInspector):

    def __init__(self, model: str, api_key: str):
        self.model = model
        self.api_key = api_key

    def inspect(self, prompt: str, image: str) -> str:
        if not image.startswith('http'):
            image = self._image_to_base64(image)

        response = dashscope.MultiModalConversation.call(
            model=self.model,
            api_key=self.api_key,
            messages=[
                {
                    'role': 'user',
                    "content": [
                        {"image": image},
                        {"type": "text", "text": prompt},
                    ],
                }
            ],
        )
        return response.output.choices[0].message.content[0]['text']

    def _image_to_base64(self, image_path: str) -> str:
        image_path = Path(image_path)

        with open(image_path, 'rb') as image_file:
            base64_image = base64.b64encode(image_file.read()).decode('utf-8')
            base64_image = f'data:image/{image_path.suffix.strip(".")};base64,{base64_image}'

        return base64_image


import base64
import time
from pathlib import Path
from imgenx.predictor.base.base_image_to_video import BaseImageToVideo
from volcenginesdkarkruntime import Ark


class DoubaoImageToVideo(BaseImageToVideo):

    def __init__(self, model: str, api_key: str):
        self.model = model
        self.client = Ark(
            base_url='https://ark.cn-beijing.volces.com/api/v3',
            api_key=api_key,
        )

    def image_to_video(self, prompt: str, first_frame: str, last_frame: str|None = None, 
                       resolution: str = '720p', ratio: str = '16:9', duration: int = 5) -> str:
        '''根据提示词和首尾帧图片生成视频。

        Args:
            prompt (str): 生成视频的提示词
            first_frame (str): 视频的首帧图片url或文件路径
            last_frame (str|None): 视频的尾图片url或文件路径，默认None
            resolution (str): 生成视频的分辨率：480p、720、1080p
            ratio (str): 生成视频的比例：16:9、4:3、1:1、3:4、9:16、21:9
            duration (int): 生成视频的时长，单位秒，支持 2~12 秒
        
        Returns:
            视频下载的url
        '''
        if not first_frame.startswith('http'):
            first_frame = self._image_to_base64(first_frame)

        if last_frame is not None and not last_frame.startswith('http'):
            last_frame = self._image_to_base64(last_frame)

        content = [
            {
                'type': 'text',
                'text': prompt.strip()
            },
            {
                'type': 'image_url',
                'image_url': {
                    'url': first_frame
                },
                'role': 'first_frame'
            }
        ]

        if last_frame is not None:
            content.append({
                'type': 'image_url',
                'image_url': {
                    'url': last_frame
                },
                'role': 'last_frame'
            })

        create_result = self.client.content_generation.tasks.create(
            model=self.model,
            content=content,
            extra_body={
                'resolution': resolution,
                'ratio': ratio,
                'duration': duration,
            }
        )
        task_id = create_result.id

        while True:
            get_result = self.client.content_generation.tasks.get(task_id=task_id)
            status = get_result.status
            if status == "succeeded":
                return get_result.content.video_url
            elif status == "failed":
                raise Exception(str(get_result.error))
            else:
                time.sleep(3)

    def _image_to_base64(self, image_path: str) -> str:
        image_path = Path(image_path)

        with open(image_path, 'rb') as image_file:
            base64_image = base64.b64encode(image_file.read()).decode('utf-8')
            base64_image = f'data:image/{image_path.suffix.strip(".")};base64,{base64_image}'

        return base64_image


import os
from typing import List, Dict
from pathlib import Path
from datetime import datetime

import requests
from dotenv import load_dotenv

from imgenx import factory, utils


load_dotenv()


def text_to_image(model: str, api_key: str, prompt: str, size: str) -> List[Dict[str, str]]:
    generator = factory.create_text_to_image(model, api_key)
    url_list = generator.text_to_image(prompt, size)
    return url_list


def image_to_image(model: str, api_key: str, prompt: str, images: List[str], size: str) -> List[Dict[str, str]]:
    generator = factory.create_image_to_image(model, api_key)
    url_list = generator.image_to_image(prompt, images, size)
    return url_list


def text_to_video(model: str, api_key: str, prompt: str,
                  resolution: str = '720p', ratio: str = '16:9', duration: int = 5) -> str:
    generator = factory.create_text_to_video(model, api_key)
    url = generator.text_to_video(prompt, resolution, ratio, duration)
    return url


def image_to_video(model: str, api_key: str, prompt: str, first_frame: str, last_frame: str|None = None,
                  resolution: str = '720p', ratio: str = '16:9', duration: int = 5) -> str:
    generator = factory.create_image_to_video(model, api_key)
    url = generator.image_to_video(prompt, first_frame, last_frame, resolution, ratio, duration)
    return url


def gen_image(prompt: str, size: str, output: str, images: List[str] = None):
    print('Generate images...')
    output = Path(output)

    if output.exists() and output.is_file():
        raise ValueError(f'Output path {output} already exists.')

    if images is not None and len(images) > 0:
        model, api_key = utils.get_provider_model_api_key('text_to_image', os.environ)

        if model is None:
            raise ValueError('Envrioment variable IMGENX_IMAGE_TO_IMAGE is empty.')

        if api_key is None:
            raise ValueError(f'Envrioment variable IMGENX_{model.split(":")[0].upper()}_API_KEY is empty.')

        url_list = image_to_image(model, api_key, prompt, images, size)
    else:
        model, api_key = utils.get_provider_model_api_key('text_to_image', os.environ)

        if model is None:
            raise ValueError('Envrioment variable IMGENX_TEXT_TO_IMAGE is empty.')

        if api_key is None:
            raise ValueError(f'Envrioment variable IMGENX_{model.split(":")[0].upper()}_API_KEY is empty.')

        url_list = text_to_image(model, api_key, prompt, size)

    if output.is_dir():
        path_list = [f'{output}/{datetime.now().strftime("%Y-%m-%d_%H:%M:%S")}_{i + 1}.png' for i in range(len(url_list))]
    elif len(url_list) == 1:
        path_list = [output]
    else:
        path_list = [f'{output.parent}/{output.stem}_{i + 1}{output.suffix if output.suffix else ".jpg"}' for i in range(len(url_list))]
    
    for url_item, path in zip(url_list, path_list):
        response = requests.get(url_item['url'])
        Path(path).write_bytes(response.content)
        print(f'Save image to {path}')


def gen_video(prompt: str, first_frame: str|None = None, last_frame: str|None = None,
              resolution: str = '720p', ratio: str = '16:9', duration: int = 5, output: str = None):
    print('Generate video...')

    if output is None:
        output = f'{datetime.now().strftime("%Y-%m-%d_%H:%M:%S")}.mp4'
    else:
        output = Path(output).with_name(f'{Path(output).stem}_{datetime.now().strftime("%Y-%m-%d_%H:%M:%S")}{Path(output).suffix}')

    if output.exists() and output.is_file():
        raise ValueError(f'Output path {output} already exists.')

    if first_frame is None and last_frame is None:
        model, api_key = utils.get_provider_model_api_key('text_to_video', os.environ)

        if model is None:
            raise ValueError('Envrioment variable IMGENX_TEXT_TO_VIDEO is empty.')

        if api_key is None:
            raise ValueError(f'Envrioment variable IMGENX_{model.split(":")[0].upper()}_API_KEY is empty.')

        url = text_to_video(model, api_key, prompt, resolution, ratio, duration)
    else:
        model, api_key = utils.get_provider_model_api_key('image_to_video', os.environ)

        if model is None:
            raise ValueError('Envrioment variable IMGENX_IMAGE_TO_VIDEO is empty.')

        if api_key is None:
            raise ValueError(f'Envrioment variable IMGENX_{model.split(":")[0].upper()}_API_KEY is empty.')

        url = image_to_video(model, api_key, prompt, first_frame, last_frame, resolution, ratio, duration)

    response = requests.get(url)
    Path(output).write_bytes(response.content)
    print(f'Save video to {output}')



from argparse import ArgumentParser


def run():
    parser = ArgumentParser()
    subparsers = parser.add_subparsers(dest='command', required=True)

    server_parser = subparsers.add_parser('server', help='启动 MCP 服务器')
    server_parser.add_argument('--transport', default='stdio', help='stdio|sse|streamable-http')
    server_parser.add_argument('--host', default='0.0.0.0', help='主机地址')
    server_parser.add_argument('--port', default=8000, type=int, help='端口')
    server_parser.add_argument('--disable_tools', nargs='+', default=None, help='禁用的工具名列表（用空格分隔）')

    image_parser = subparsers.add_parser('image', help='生成图片')
    image_parser.add_argument('prompt', help='生成图片的提示词')
    image_parser.add_argument('--images', nargs='+', default=None, help='输入图片路径列表')
    image_parser.add_argument('--size', default='2K', help='生成图像的分辨率或宽高像素值，分辨率可选值：1K、2K、4K，宽高像素可选值：2048x2048、2304x1728、1728x2304、2560x1440、1440x2560、2496x1664、1664x2496、3024x1296')
    image_parser.add_argument('--output', default='imgenx.jpg', help='输出文件或目录路径')

    video_parser = subparsers.add_parser('video', help='生成视频')
    video_parser.add_argument('prompt', help='生成视频的提示词')
    video_parser.add_argument('--first_frame', default=None, help='输入视频的第一帧路径')
    video_parser.add_argument('--last_frame', default=None, help='输入视频的最后一帧路径')
    video_parser.add_argument('--resolution', default='720p', help='生成视频的分辨率，可选值：480p、720、1080p')
    video_parser.add_argument('--ratio', default='16:9', help='生成视频的宽高比，可选值：16:9、4:3、1:1、3:4、9:16、21:9')
    video_parser.add_argument('--duration', default=5, type=int, help='生成视频的时长，单位秒')
    video_parser.add_argument('--output', default='imgenx.mp4', help='输出文件路径')

    args = parser.parse_args()

    if args.command == 'server':
        from imgenx.server import mcp

        if args.disable_tools:
            for tool in args.disable_tools:
                mcp.remove_tool(tool)

        if args.transport == 'stdio':
            mcp.run(transport='stdio')
        else:
            mcp.run(transport=args.transport, host=args.host, port=args.port)
    elif args.command == 'image':
        from imgenx import script
        script.gen_image(prompt=args.prompt, size=args.size, output=args.output, images=args.images)
    elif args.command == 'video':
        from imgenx import script
        script.gen_video(prompt=args.prompt, first_frame=args.first_frame, last_frame=args.last_frame,
                   resolution=args.resolution, ratio=args.ratio, duration=args.duration, output=args.output)
    else:
        raise ValueError(f'Unknown command: {args.command}')


if __name__ == '__main__':
    run()


from pathlib import Path
from functools import cache
from importlib import import_module


@cache
def create_text_to_image(model: str, api_key: str):
    return create_predictor(model, api_key, 'text_to_image')


@cache
def create_image_to_image(model: str, api_key: str):
    return create_predictor(model, api_key, 'image_to_image')


@cache
def create_text_to_video(model: str, api_key: str):
    return create_predictor(model, api_key, 'text_to_video')


@cache
def create_image_to_video(model: str, api_key: str):
    return create_predictor(model, api_key, 'image_to_video')


@cache
def create_image_inspector(model: str, api_key: str):
    return create_predictor(model, api_key, 'image_inspector')


def create_predictor(model: str, api_key: str, role: str):
    provider, model = model.split(':')
    provider = provider.lower()

    if provider not in get_providers():
        raise ValueError(f'Provider {provider} not found.')

    predictor_package = f'imgenx.predictor.generators.{provider}.{role.lower()}'
    role_parts = [p.capitalize() for p in role.split('_')]
    predictor_class = f'{provider.capitalize()}{"".join(role_parts)}'

    try:
        package = import_module(predictor_package)
        predictor = getattr(package, predictor_class)
    except (ImportError, AttributeError):
        raise ValueError(f'Provider {provider} not found.')

    return predictor(model, api_key)


@cache
def get_providers():
    root = Path(__file__).parent / 'predictor/generators'
    return [p.name for p in root.iterdir() if p.is_dir()]



def get_provider_model_api_key(task, headers, env):
    task = task.strip()

    try:
        headers = {key.lower(): value for key, value in headers.items()}
        env = {key.lower(): value for key, value in env.items()}

        provider_model_name = f'imgenx_{task}'
        provider_model = headers.get(provider_model_name, env.get(provider_model_name))
        provider, model = provider_model.split(':')
    except Exception as e:
        provider = None
    
    api_key = headers.get(f'imgenx_{provider}_api_key', env.get(f'imgenx_{provider}_api_key'))

    return provider_model, api_key




from typing import List, Dict
from abc import ABC, abstractmethod


class BaseTextToImage(ABC):

    @abstractmethod
    def __init__(self, model: str, api_key: str):
        raise NotImplementedError

    @abstractmethod
    def text_to_image(self, prompt: str, size: str):
        raise NotImplementedError



import dashscope
from imgenx.predictor.base.base_text_to_image import BaseTextToImage


class AliyunTextToImage(BaseTextToImage):

    def __init__(self, model: str, api_key: str):
        self.model = model
        self.api_key = api_key

    def text_to_image(self, prompt: str, size: str):
        '''根据提示词生成图片

        Args:
            prompt (str): 图片描述提示词
            size (str): 图片分辨率，1664*928: 16:9, 1472*1140: 4:3, 1328*1328(默认值): 1:1, 1140*1472: 3:4, 928*1664: 9:16
        
        Returns:
            图片url列表
        '''
        message = [
            {'role': 'user', 'content': [{'text': prompt}]}
        ]
        response = dashscope.MultiModalConversation.call(
            model=self.model,
            api_key=self.api_key,
            messages=message,
            size=size.replace('x', '*'),
            watermark=False
        )

        result = []
        for item in response.output.choices[0].message.content:
            result.append(item['image'])
        return result
